package com.jpsoft.picc.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinition;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinitionLimit;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionLimitService;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("/insuranceDefinition")
@Api(description = "险种信息")
public class InsuranceDefinitionController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsuranceDefinitionService insuranceDefinitionService;

    @Autowired
    private InsuranceDefinitionLimitService insuranceDefinitionLimitService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<InsuranceDefinition> create(){
        MessageResult<InsuranceDefinition> msgResult = new MessageResult<>();

        InsuranceDefinition insuranceDefinition = new InsuranceDefinition();

        msgResult.setData(insuranceDefinition);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<InsuranceDefinition> add(@RequestBody InsuranceDefinition insuranceDefinition,@RequestAttribute String subject){
        MessageResult<InsuranceDefinition> msgResult = new MessageResult<>();

        try {
            insuranceDefinition.setId(UUID.randomUUID().toString());
            insuranceDefinition.setDelFlag(false);
            insuranceDefinition.setCreateBy(subject);
            insuranceDefinition.setCreateTime(new Date());
            
            int affectCount = insuranceDefinitionService.insert(insuranceDefinition);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceDefinition);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<InsuranceDefinition> edit(@PathVariable("id") String id){
        MessageResult<InsuranceDefinition> msgResult = new MessageResult<>();

        try {
            InsuranceDefinition insuranceDefinition = insuranceDefinitionService.get(id);

            if (insuranceDefinition != null) {
                msgResult.setResult(true);
                msgResult.setData(insuranceDefinition);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<InsuranceDefinition> update(@RequestBody InsuranceDefinition insuranceDefinition,@RequestAttribute String subject){
        MessageResult<InsuranceDefinition> msgResult = new MessageResult<>();

        try {
            insuranceDefinition.setUpdateBy(subject);
            insuranceDefinition.setUpdateTime(new Date());
            
            int affectCount = insuranceDefinitionService.update(insuranceDefinition);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceDefinition);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsuranceDefinition insuranceDefinition = insuranceDefinitionService.get(id);
            insuranceDefinition.setDelFlag(true);
            insuranceDefinition.setUpdateBy(subject);
            insuranceDefinition.setUpdateTime(new Date());

            int affectCount = insuranceDefinitionService.update(insuranceDefinition);

            if (affectCount > 0) {

                List<InsuranceDefinitionLimit> insuranceDefinitionLimitList = insuranceDefinitionLimitService.findByDefinitionId(insuranceDefinition.getId());
                for (InsuranceDefinitionLimit insuranceDefinitionLimit:insuranceDefinitionLimitList) {
                    insuranceDefinitionLimit.setDelFlag(true);
                    insuranceDefinitionLimit.setUpdateBy(subject);
                    insuranceDefinitionLimit.setUpdateTime(new Date());
                    insuranceDefinitionLimitService.update(insuranceDefinitionLimit);
                }

                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                InsuranceDefinition insuranceDefinition = insuranceDefinitionService.get(id);
                insuranceDefinition.setDelFlag(true);
                insuranceDefinition.setUpdateBy(subject);
                insuranceDefinition.setUpdateTime(new Date());

                affectCount += insuranceDefinitionService.update(insuranceDefinition);

                //如果下面有限额，限额一并删除
                List<InsuranceDefinitionLimit> insuranceDefinitionLimitList = insuranceDefinitionLimitService.findByDefinitionId(insuranceDefinition.getId());

                for (InsuranceDefinitionLimit insuranceDefinitionLimit:insuranceDefinitionLimitList) {
                    insuranceDefinitionLimit.setDelFlag(true);
                    insuranceDefinitionLimit.setUpdateBy(subject);
                    insuranceDefinitionLimit.setUpdateTime(new Date());
                    insuranceDefinitionLimitService.update(insuranceDefinitionLimit);
                }
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="name",value = "名称",paramType = "query")
    })
    public MessageResult<Map> pageList(
            String name,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%" + name + "%");
        }

        Page<InsuranceDefinition> page = insuranceDefinitionService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="所有险种列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    public MessageResult<List<InsuranceDefinition>> list(){

        MessageResult<List<InsuranceDefinition>> msgResult = new MessageResult<>();

        List<InsuranceDefinition> list = insuranceDefinitionService.list();

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }
}
