package com.jpsoft.picc.modules.business.controller;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.github.pagehelper.Page;
import com.github.pagehelper.util.StringUtil;
import com.jpsoft.picc.modules.base.entity.*;
import com.jpsoft.picc.modules.base.service.*;
import com.jpsoft.picc.modules.business.entity.*;
import com.jpsoft.picc.modules.business.service.*;
import com.jpsoft.picc.modules.common.config.WeixinConfig;
import com.jpsoft.picc.modules.common.constant.PolicyStatus;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.utils.*;
import com.jpsoft.picc.modules.sys.entity.DataDictionary;
import com.jpsoft.picc.modules.sys.entity.User;
import com.jpsoft.picc.modules.sys.service.DataDictionaryService;
import com.jpsoft.picc.modules.sys.service.UserRoleService;
import com.jpsoft.picc.modules.sys.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.Months;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/insurancePolicy")
@Api(description = "每月投保单")
public class InsurancePolicyController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsurancePolicyService insurancePolicyService;

    @Autowired
    private InsuranceAgentService insuranceAgentService;

    @Autowired
    private InsurancePolicyRecordService insurancePolicyRecordService;

    @Autowired
    private InsuranceApplicationService insuranceApplicationService;

    @Autowired
    private ApplicationPolicyService applicationPolicyService;

    @Autowired
    private InsurancePolicyMemberService insurancePolicyMemberService;

    @Autowired
    private InsuranceDefinitionLimitService insuranceDefinitionLimitService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private CompanyUserService companyUserService;

    @Autowired
    private InsuranceJobsService insuranceJobsService;

    @Autowired
    private InsuranceDefinitionService insuranceDefinitionService;

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private TemplateMessageService templateMessageService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<InsurancePolicy> create(){
        MessageResult<InsurancePolicy> msgResult = new MessageResult<>();

        InsurancePolicy insurancePolicy = new InsurancePolicy();

        msgResult.setData(insurancePolicy);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<InsurancePolicy> add(@RequestBody InsurancePolicy insurancePolicy,@RequestAttribute String subject){
        MessageResult<InsurancePolicy> msgResult = new MessageResult<>();

        try {
            insurancePolicy.setId(UUID.randomUUID().toString());
            insurancePolicy.setDelFlag(false);
            insurancePolicy.setCreateBy(subject);
            insurancePolicy.setCreateTime(new Date());

            int affectCount = insurancePolicyService.insert(insurancePolicy);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicy);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加issued失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<InsurancePolicy> edit(@PathVariable("id") String id){
        MessageResult<InsurancePolicy> msgResult = new MessageResult<>();

        try {
            InsurancePolicy insurancePolicy = insurancePolicyService.get(id);

            if (insurancePolicy != null) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicy);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<InsurancePolicy> update(@RequestBody InsurancePolicy insurancePolicy,@RequestAttribute String subject){
        MessageResult<InsurancePolicy> msgResult = new MessageResult<>();

        try {
            insurancePolicy.setUpdateBy(subject);
            insurancePolicy.setUpdateTime(new Date());

            int affectCount = insurancePolicyService.update(insurancePolicy);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicy);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsurancePolicy insurancePolicy = insurancePolicyService.get(id);
            insurancePolicy.setDelFlag(true);
            insurancePolicy.setUpdateBy(subject);
            insurancePolicy.setUpdateTime(new Date());

            int affectCount = insurancePolicyService.update(insurancePolicy);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                InsurancePolicy insurancePolicy = insurancePolicyService.get(id);
                insurancePolicy.setDelFlag(true);
                insurancePolicy.setUpdateBy(subject);
                insurancePolicy.setUpdateTime(new Date());

                affectCount += insurancePolicyService.update(insurancePolicy);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<InsurancePolicy> page = insurancePolicyService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="读取附件")
    @RequestMapping(value = "attachmentList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="id",value = "每月投保单ID",required = true,paramType = "query")
    })
    public MessageResult<List<Map<String,Object>>> attachmentList(
            String id, @RequestAttribute String subject,
            HttpServletRequest request){
        //当前用户ID
        MessageResult<List<Map<String,Object>>> msgResult = new MessageResult<List<Map<String,Object>>>();

        InsurancePolicy insurancePolicy = insurancePolicyService.get(id);

//        String file = insurancePolicy.getInsurancePolicyFile();
        List<Map<String,Object>> fileList = new ArrayList<Map<String,Object>>();
//        if(StringUtils.isNotEmpty(file)){
//            String[] files = file.split(";");
//
//            int i=1;
//
//            for (String str:files) {
//                Map<String,Object> map = new HashMap<String,Object>();
//                map.put("index",i);
//                map.put("file",str);
//                String name = "";
//                if(StringUtils.isNotEmpty(str)){
//                    name = str.substring(str.lastIndexOf("/")+1);
//                }
//                map.put("name",name);
//                fileList.add(map);
//                i++;
//            }
//        }

        String path = request.getContextPath();
        String basePath = request.getScheme()+"://"+request.getServerName()+":"+request.getServerPort()+path+"/";

        List<InsuranceApplication> applicationList = applicationPolicyService.findApplicationByPolicyId(insurancePolicy.getId());

        if (applicationList.size()>0) {
            InsuranceApplication insuranceApplication = applicationList.get(0);

            Company company = companyService.get(insuranceApplication.getCompanyId());

            Map<String,Object> file1 = new HashMap<>();

            String url = company.getUsccFiles();

            if(StringUtils.isNotEmpty(url)) {
                file1.put("index", "1");
                file1.put("name", "营业执照" + url.substring(url.lastIndexOf(".")));
                file1.put("url", url);

                fileList.add(file1);
            }

            Map<String,Object> file2 = new HashMap<>();
            file2.put("index","2");
            file2.put("name","身份证.zip");
            file2.put("url",basePath + "pub/attachment/downloadIdCardZip?policyId=" + id);

            fileList.add(file2);
        }

        msgResult.setResult(true);
        msgResult.setData(fileList);

        return msgResult;
    }

    @ApiOperation(value="读取推广信息")
    @RequestMapping(value = "loadInsuranceAgent",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="id",value = "每月投保单ID",required = true,paramType = "query")
    })
    public MessageResult<InsuranceAgent> loadInsuranceAgent(String id, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<InsuranceAgent> msgResult = new MessageResult<>();

        InsurancePolicy insurancePolicy = insurancePolicyService.get(id);

        String agentId = insurancePolicy.getAgentId();

        InsuranceAgent insuranceAgent = insuranceAgentService.get(agentId);

        msgResult.setResult(true);
        msgResult.setData(insuranceAgent);

        return msgResult;
    }


    @ApiOperation(value="人才超市初审")
    @RequestMapping(value = "firstApproval",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query")
    })
    public MessageResult<Integer> firstApproval(String applicationId,String policyId,String processStatus,String opinion, @RequestAttribute String subject){
        //当前用户ID
        System.out.println(subject);

        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);
            PolicyStatus srcPolicyStatus = PolicyStatus.valueOf(Integer.valueOf(insurancePolicy.getStatus()));

            PolicyStatus destPolicyStatus;

            if (processStatus.equals("正常")) {
                //30
                destPolicyStatus = PolicyStatus.PendingRetrial;
            } else {
                //15
                destPolicyStatus = PolicyStatus.Back;
            }

            int affectCount = approval(applicationId, policyId, processStatus, opinion, destPolicyStatus.getValue() + "", subject);

            if (processStatus.equals("正常")) {
                //修改花名册人员状态为初审
                check(policyId, "1", subject);
            }

            //发送微信通知
            templateMessageService.sendTemplateMessage(policyId, srcPolicyStatus, destPolicyStatus);

            msgResult.setResult(true);
            msgResult.setData(affectCount);
        }
        catch (Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    private void sendWeixinMessage(String policyId,PolicyStatus srcPolicyStatus, PolicyStatus destPolicyStatus) {
        templateMessageService.sendTemplateMessage(policyId, srcPolicyStatus, destPolicyStatus);
    }

    @ApiOperation(value="PICC复审")
    @RequestMapping(value = "secondApproval",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query")
    })
    public MessageResult<Integer> secondApproval(String applicationId,String policyId,String processStatus,String opinion, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);
            PolicyStatus srcPolicyStatus = PolicyStatus.valueOf(Integer.valueOf(insurancePolicy.getStatus()));

            PolicyStatus destPolicyStatus;

            if (processStatus.equals("正常")) {
                //40
                if (insurancePolicy != null) {
                    int no = insurancePolicy.getNo();
                    if (no > 1) {
                        //待制单
                        destPolicyStatus = PolicyStatus.PendingMakePolicy;
                    } else {
                        //待缴费
                        destPolicyStatus = PolicyStatus.PendingPay;
                    }
                } else {
                    throw new Exception("下月投保单不存在");
                }
            } else if("资料不齐".equals(processStatus)){
                if (insurancePolicy != null) {
                    //材料不齐
                    //sz
                    destPolicyStatus = PolicyStatus.Back;

                } else {
                    throw new Exception("下月投保单不存在");
                }
            }else {
                //20
                destPolicyStatus = PolicyStatus.PendingTrial;
            }

            int affectCount = approval(applicationId, policyId, processStatus, opinion,destPolicyStatus.getValue() + "", subject);

            if (processStatus.equals("正常")) {
                //修改花名册人员状态为初审
                check(policyId, "2", subject);
            }

            //发送微信通知
            templateMessageService.sendTemplateMessage(policyId,srcPolicyStatus, destPolicyStatus);

            msgResult.setResult(true);
            msgResult.setData(affectCount);
        }
        catch (Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }


    @ApiOperation(value="PICC出单")
    @RequestMapping(value = "issued",method = RequestMethod.POST)
    @Transactional(rollbackFor = Exception.class,propagation = Propagation.REQUIRED)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query"),
            @ApiImplicitParam(name="insurancePolicyFile",value = "投保单附件",required = true,paramType = "query"),
            @ApiImplicitParam(name="electronicInvoiceFile",value = "发票附件",required = true,paramType = "query")
    })
    public MessageResult<Integer> issued(String applicationId,String policyId,String processStatus,String opinion,
                                         String insurancePolicyFile,String electronicInvoiceFile,
                                         @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);
            PolicyStatus srcPolicyStatus = PolicyStatus.valueOf(Integer.valueOf(insurancePolicy.getStatus()));

            PolicyStatus destPolicyStatus;

            if (processStatus.equals("正常")) {
                //70
                destPolicyStatus = PolicyStatus.SendOutPolicy;
            } else {
                //50
                destPolicyStatus = PolicyStatus.PendingMakePolicy;
            }

            InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);

            Date now = new Date();

            //首月投保单
            if(insurancePolicy.getNo().equals(1)){
                insuranceApplication.setFinishTime(now);
                insuranceApplicationService.update(insuranceApplication);
            }

            if (StringUtils.isNotEmpty(insurancePolicyFile)) {
                insurancePolicy.setInsurancePolicyFile(insurancePolicyFile);
            }

            //发票附件
            if (StringUtils.isNotEmpty(electronicInvoiceFile)) {
                insurancePolicy.setElectronicInvoiceFile(electronicInvoiceFile);
            }

            insurancePolicy.setFinishTime(now);
            insurancePolicyService.update(insurancePolicy);

            //todo 审批并生成下月投保单
            approveAndGenPolicyInTrans(applicationId, policyId,processStatus,opinion,destPolicyStatus,subject);

            //发送微信通知
            templateMessageService.sendTemplateMessage(policyId,srcPolicyStatus, destPolicyStatus);

            msgResult.setResult(true);
        }
        catch (Exception ex) {
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());

            //在实际开发中，我们往往需要在方法中进行异常的捕获，从而对异常进行判断，为客户端返回提示信息。
            //但是此时由于异常的被捕获，导致事务的回滚没有被触发，导致事务的失败。
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }

        return msgResult;
    }

    @ApiOperation(value="人才超市与PICC转账")
    @RequestMapping(value = "transfer",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="accountBank",value = "入账银行",required = true,paramType = "query"),
            @ApiImplicitParam(name="issuingBank",value = "出账银行",required = true,paramType = "query"),
            @ApiImplicitParam(name="singlNumber",value = "单号",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query"),
            @ApiImplicitParam(name="agentId",value = "保险代理人ID",required = true,paramType = "query")
    })
    public MessageResult<Integer> transfer(String applicationId,String policyId,
                                           String accountBank,String issuingBank,String singlNumber,
                                           String processStatus,String opinion,String agentId, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
//        String status ="";
            InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);
            PolicyStatus srcPolicyStatus = PolicyStatus.valueOf(Integer.valueOf(insurancePolicy.getStatus()));

            PolicyStatus destPolicyStatus;

            if (processStatus.equals("正常")) {
                //60
                destPolicyStatus = PolicyStatus.PendingOutPolicy;
            } else {
                //40
                destPolicyStatus = PolicyStatus.PendingPay;
            }

            InsurancePolicy item = insurancePolicyService.get(policyId);

            item.setAccountBank(accountBank);
            item.setIssuingBank(issuingBank);
            item.setSinglNumber(singlNumber);
            item.setTransferTime(new Date());
            item.setAgentId(agentId);

            int affectCount = insurancePolicyService.update(item);

            int affectCount1 = 0;

            if (affectCount > 0) {
                affectCount1 = approval(applicationId, policyId, processStatus, opinion, destPolicyStatus.getValue() + "", subject);
            }

            //发送微信通知
            templateMessageService.sendTemplateMessage(policyId, srcPolicyStatus, destPolicyStatus);

            msgResult.setResult(true);
            msgResult.setData(affectCount1);
        }
        catch (Exception ex){
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


        @ApiOperation(value="回退转单通用")
        @RequestMapping(value = "rollbackOrder",method = RequestMethod.POST)
        @ApiImplicitParams({
                @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
                @ApiImplicitParam(name="policyId",value = "每月投保单ID",paramType = "query"),
                @ApiImplicitParam(name="status",value = "转单步骤",required = true,paramType = "query")
        })
        public MessageResult<Integer> rollbackOrder(String applicationId,String policyId,String status, @RequestAttribute String subject){
            //当前用户ID
            System.out.println(subject);

            MessageResult<Integer> msgResult = new MessageResult<>();

            try {
                InsurancePolicy insurancePolicy;

                if(StringUtils.isNotEmpty(policyId)){
                    insurancePolicy = insurancePolicyService.get(policyId);
                }
                else{
                    insurancePolicy = applicationPolicyService.findFirstPolicyByApplicationId(applicationId);
                    policyId = insurancePolicy.getId();
                }

                PolicyStatus srcPolicyStatus = PolicyStatus.valueOf(Integer.valueOf(insurancePolicy.getStatus()));

                PolicyStatus destPolicyStatus = PolicyStatus.valueOf(Integer.valueOf(status));

                int affectCount = approval(applicationId, policyId, "回退", "", destPolicyStatus.getValue()+ "", subject);

                //发送微信通知
                templateMessageService.sendTemplateMessage(policyId, srcPolicyStatus, destPolicyStatus);

                msgResult.setResult(true);
                msgResult.setData(affectCount);
            }
            catch (Exception ex){
                logger.error(ex.getMessage(),ex);
                msgResult.setMessage(ex.getMessage());
            }

            return msgResult;
        }


        private Integer approval(String applicationId,String policyId,String processStatus,String opinion,String status,String subject){
        int affectCount = 0;

        InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);

        InsurancePolicy insurancePolicy;

        if(StringUtils.isNotEmpty(policyId)){
            insurancePolicy = insurancePolicyService.get(policyId);
        }
        else{
            List<ApplicationPolicy> applicationPolicyList = applicationPolicyService.findByApplicationId(applicationId);
            ApplicationPolicy applicationPolicy = applicationPolicyList.get(0);
            insurancePolicy = applicationPolicy.getPolicy();
        }

        String curStatus = insurancePolicy.getStatus();

        //首月投保单
        if(insurancePolicy.getNo().equals(1)){
            insuranceApplication.setStatus(status);
            insuranceApplicationService.update(insuranceApplication);
        }

        insurancePolicy.setProcessStatus(processStatus);
        insurancePolicy.setStatus(status);
        insurancePolicy.setUpdateBy(subject);
        insurancePolicy.setUpdateTime(new Date());

        int affectCount1 = insurancePolicyService.update(insurancePolicy);

        if(affectCount1 > 0){
            InsurancePolicyRecord insurancePolicyRecord = new InsurancePolicyRecord();
            insurancePolicyRecord.setPolicyId(insurancePolicy.getId());
            insurancePolicyRecord.setStatus(curStatus);
            insurancePolicyRecord.setOpinion(opinion);
            insurancePolicyRecord.setProcessStatus(processStatus);
            insurancePolicyRecord.setId(UUID.randomUUID().toString());
            insurancePolicyRecord.setDelFlag(false);
            insurancePolicyRecord.setCreateBy(subject);
            insurancePolicyRecord.setCreateTime(new Date());

            affectCount = insurancePolicyRecordService.insert(insurancePolicyRecord);
        }

        return affectCount;
    }

    private void approveAndGenPolicyInTrans(String applicationId, String policyId,String processStatus,
                                       String opinion,PolicyStatus policyStatus,
                                       String subject) throws Exception {
        approval(applicationId, policyId, processStatus, opinion, policyStatus.getValue() + "", subject);

        if (!processStatus.equals("正常")) {
            return;
        }

        //修改本月投保单状态为已出单
        InsurancePolicy srcPolicy = insurancePolicyService.get(policyId);

        InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);

        //如果是最后一个月则不再生成投保单
        if(!srcPolicy.getEndTime().before(insuranceApplication.getEndTime())) {
            return;
        }

        InsurancePolicy destPolicy = new InsurancePolicy();

        PojoUtils.map(srcPolicy,destPolicy);

        DateTime startTime = new DateTime(srcPolicy.getEndTime());
        startTime = startTime.plusDays(1);

        destPolicy.setId(UUID.randomUUID().toString());
        destPolicy.setNo(srcPolicy.getNo()+1);

        DecimalFormat df = new DecimalFormat("00");

        String policyNo = insuranceApplication.getPolicyNo();
        String[] policyNoArr = policyNo.split("-");

        destPolicy.setPolicyNo(policyNoArr[0] + "-" + df.format(destPolicy.getNo()));

        destPolicy.setStartTime(startTime.toDate());
        destPolicy.setEndTime(startTime.plusMonths(1).plusDays(-1).toDate());

        destPolicy.setEffectiveDate(startTime.toString("yyyyMM"));
        destPolicy.setStatus(PolicyStatus.Draft.getValue() + "");
        destPolicy.setCreateTime(new Date());
        destPolicy.setCreateBy(subject);
        destPolicy.setUpdateBy(null);
        destPolicy.setUpdateTime(null);
        destPolicy.setFinishTime(null);
        destPolicy.setInsurancePolicyFile(null);
        destPolicy.setSinglNumber(null);

        //出账银行、入账银行、保险代理人设置值暂没去掉，保留到下个月。

        //todo 如果有则判断关联当前投保单的下月投保记录是否存在
        boolean exist = insurancePolicyService.existByApplicationIdAndEffectiveDate(applicationId, destPolicy.getEffectiveDate());

        if (!exist) {
            insurancePolicyService.insert(destPolicy);

            //todo 关联投保申请单及下月月投保单
            ApplicationPolicy applicationPolicy = new ApplicationPolicy();

            applicationPolicy.setId(UUID.randomUUID().toString());
            applicationPolicy.setApplicationId(applicationId);
            applicationPolicy.setPolicyId(destPolicy.getId());

            applicationPolicyService.insert(applicationPolicy);

            //todo 关联参保人员
            List<InsurancePolicyMember> memberList = insurancePolicyMemberService.findByPolicyId(policyId);

            for (InsurancePolicyMember srcMember : memberList) {
                InsurancePolicyMember destMember = new InsurancePolicyMember();

                destMember.setId(UUID.randomUUID().toString());
                destMember.setPolicyId(destPolicy.getId());
                destMember.setMemberId(srcMember.getMemberId());
                destMember.setDelFlag(false);
                destMember.setCreateBy(subject);
                destMember.setCreateTime(new Date());
                destMember.setStatus(srcMember.getStatus());

                insurancePolicyMemberService.insert(destMember);
            }

            insurancePolicyService.updatePolicyNumAndAmount(destPolicy,subject);
        }
        else{
            throw new Exception(destPolicy.getEffectiveDate() + "投保单已存在!");
        }
    }

    private void check(String policyId,String status,String subject){
        List<InsurancePolicyMember> insurancePolicyMemberList = insurancePolicyMemberService.findByPolicyId(policyId);

        for (InsurancePolicyMember insurancePolicyMember:insurancePolicyMemberList) {
            String statusMember = insurancePolicyMember.getStatus();
            if(StringUtils.isEmpty(statusMember)||statusMember.equals("0")){
                insurancePolicyMember.setStatus(status);
                insurancePolicyMember.setUpdateBy(subject);
                insurancePolicyMember.setUpdateTime(new Date());
                insurancePolicyMemberService.update(insurancePolicyMember);
            }
            else if(statusMember.equals("1")&&status.equals("2")){
                insurancePolicyMember.setStatus(status);
                insurancePolicyMember.setUpdateBy(subject);
                insurancePolicyMember.setUpdateTime(new Date());
                insurancePolicyMemberService.update(insurancePolicyMember);
            }
        }
    }


    @ApiOperation(value="获取投保信息")
    @PostMapping("insureDetail")
    @ApiImplicitParams({
            @ApiImplicitParam(name="policyId",value = "下月投保单ID",required = true,paramType = "query")
    })
    public MessageResult<InsuranceApplication> insureDetail(String policyId){
        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        try {
            InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);

            if (insurancePolicy != null) {
                List<InsuranceApplication> applicationList = applicationPolicyService.findApplicationByPolicyId(policyId);

                //首月投保
                InsuranceApplication insuranceApplication = applicationList.get(0);

                insuranceApplication.setInsurancePolicy(insurancePolicy);

                //营业性质
//                if (StringUtils.isNotEmpty(insuranceApplication.getBusinessNature())){
//                    String businessNatureName = dataDictionaryService.findNameByCatalogNameAndValue("营业性质",insuranceApplication.getBusinessNature());
//                    insuranceApplication.setBusinessNatureN(businessNatureName);
//                }

                msgResult.setResult(true);
                msgResult.setData(insuranceApplication);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="获取承保信息")
    @RequestMapping(value = "acceptDetail",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query")
    })
    public MessageResult<Map> acceptDetail(String policyId){

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> map = new HashMap<String,Object>();

        //每月投保单
        InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);

        //险种
        InsuranceDefinition insuranceDefinition = insuranceDefinitionService.get(insurancePolicy.getDefinitionId());

        String definitionId = insuranceDefinition.getId();

        map.put("id",insuranceDefinition.getId());
        map.put("name",insuranceDefinition.getName());

        List<InsuranceDefinitionLimit> insuranceDefinitionLimitList = insuranceDefinitionLimitService.findByDefinitionId(definitionId);

        List<Map<String,Object>> mapList = new ArrayList<Map<String,Object>>();


        Map<String,Object> varsMap = new HashMap<>();
        varsMap.put("num",insurancePolicy.getNumber());

        for(InsuranceDefinitionLimit  insuranceDefinitionLimit:insuranceDefinitionLimitList){
            Map<String,Object> map1 = new HashMap<String,Object>();
            map1.put("id",insuranceDefinitionLimit.getId());
            map1.put("name",insuranceDefinitionLimit.getName());

            //限额这里可能是公式，需要带入人数
            String limit = VelocityHelper.format(insuranceDefinitionLimit.getLimit(),varsMap);
            map1.put("limit",limit);

            String unit = VelocityHelper.format(insuranceDefinitionLimit.getUnit(),varsMap);
            map1.put("unit",unit);

            mapList.add(map1);
        }

        map.put("insuranceDefinitionLimitList",mapList);

        List<InsuranceJobs> InsuranceJobsList = insuranceJobsService.findByDefinitionId(insurancePolicy.getDefinitionId());

        List<Map<String,Object>> mapList1 = new ArrayList<Map<String,Object>>();

        DateTime startTime = new DateTime(insurancePolicy.getStartTime());
        map.put("startTime",startTime.toString("yyyy-MM-dd"));

        DateTime endTime = new DateTime(insurancePolicy.getEndTime());
        map.put("endTime",endTime.toString("yyyy-MM-dd"));

        //月份
        int months = 1;

        BigDecimal totalAmount = new BigDecimal(0);

        for (InsuranceJobs insuranceJobs:InsuranceJobsList) {
            Map<String,Object> map1 = new HashMap<String,Object>();
            String jobId = insuranceJobs.getJobsId();
            map1.put("id",jobId);
            map1.put("name",insuranceJobs.getJobs().getName());
            map1.put("content",insuranceJobs.getJobs().getContent());
            map1.put("charges",insuranceJobs.getCharges());
            map1.put("chargesY",insuranceJobs.getCharges().multiply(new BigDecimal(12)));
            long memberCount = insurancePolicyMemberService.countByPolicyIdAndJobsId(policyId,insuranceJobs.getJobsId());
            long number = Long.valueOf(memberCount).intValue();
            map1.put("number",number);
            BigDecimal money = insuranceJobs.getCharges().multiply(new BigDecimal(months)).multiply(new BigDecimal(number));
            map1.put("money",money);

            totalAmount = totalAmount.add(money);
            mapList1.add(map1);
        }

        map.put("totalAmount",totalAmount);

        String cnMontrayUnit = NumberToCN.number2CNMontrayUnit(totalAmount);
        map.put("cnMontrayUnit",cnMontrayUnit);

        map.put("jobsList",mapList1);

        msgResult.setResult(true);
        msgResult.setData(map);

        return msgResult;

    }


    @ApiOperation(value="历史投保单列表或者佣金是否入录列表")
    @RequestMapping(value = "insuranceHistoryPageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="definitionId",value = "险种ID",paramType = "query"),
            @ApiImplicitParam(name="companyName",value = "公司名称",paramType = "query"),
            @ApiImplicitParam(name="promoter",value = "推广员账号",paramType = "query"),
            @ApiImplicitParam(name="agentId",value = "保险代理人ID",paramType = "query"),
            @ApiImplicitParam(name="startCreat",value = "创建开始时间",paramType = "query"),
            @ApiImplicitParam(name="endCreat",value = "创建结束时间",paramType = "query"),
            @ApiImplicitParam(name="startFinish",value = "出单开始时间",paramType = "query"),
            @ApiImplicitParam(name="endFinish",value = "出单结束时间",paramType = "query"),
            @ApiImplicitParam(name="isCommission",value = "是否录入佣金（0:否，1:是，不传查询全部）",paramType = "query"),
            @ApiImplicitParam(name="startTime",value = "投保开始时间",paramType = "query"),
            @ApiImplicitParam(name="endTime",value = "投保结束时间",paramType = "query")
    })
    public MessageResult<Map> insuranceHistoryPageList(
            String definitionId,String companyName,String promoter,String agentId,
            String startCreat,String endCreat,
            String startFinish,String endFinish,
            Boolean isCommission,String startTime,String endTime,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.create_time","desc"));

        searchParams.put("status0",PolicyStatus.Expired.getValue());

        searchParams.put("status1",PolicyStatus.SendOutPolicy.getValue());

        if (StringUtils.isNotEmpty(definitionId)) {
            searchParams.put("definitionId",definitionId);
        }
        if (StringUtils.isNotEmpty(companyName)) {
            searchParams.put("companyName","%"+companyName+"%");
        }
        if (StringUtils.isNotEmpty(promoter)) {
            searchParams.put("promoter","%"+promoter+"%");
        }
        if (StringUtils.isNotEmpty(agentId)) {
            searchParams.put("agentId",agentId);
        }
        if (StringUtils.isNotEmpty(startCreat)) {
            searchParams.put("startCreat",startCreat);
        }
        if (StringUtils.isNotEmpty(endCreat)) {
            searchParams.put("endCreat",endCreat);
        }
        if (StringUtils.isNotEmpty(startFinish)) {
            searchParams.put("startFinish",startFinish);
        }
        if (StringUtils.isNotEmpty(endFinish)) {
            searchParams.put("endFinish",endFinish);
        }

        if (isCommission!=null) {
            if(isCommission){
                searchParams.put("isCommission",true);
            }
            else{
                searchParams.put("isCommission",false);
            }

        }

        if (StringUtils.isNotEmpty(startTime)) {
            searchParams.put("startTime",startTime);
        }
        if (StringUtils.isNotEmpty(endTime)) {
            searchParams.put("endTime",endTime);
        }

        Page<InsurancePolicy> page = insurancePolicyService.hisPageSearch(searchParams,pageIndex,pageSize,sortList);

        for (InsurancePolicy insurancePolicy:page) {
            if(insurancePolicy.getCommission()!=null) {
                List<DataDictionary> dataDictionaryList = dataDictionaryService.findByCatalogName("佣金分成");

                if (dataDictionaryList.size() > 0) {

                    DataDictionary item = dataDictionaryList.get(0);

                    BigDecimal value = new BigDecimal(item.getValue());
                    BigDecimal bili = new BigDecimal("0.01");
                    BigDecimal fencheng = value.multiply(bili);
                    BigDecimal commission = insurancePolicy.getCommission();
                    BigDecimal finalFee = commission.multiply(fencheng);

                    insurancePolicy.setPromoterFee(finalFee);


                }

                if (dataDictionaryList.size() > 1) {
                    DataDictionary item = dataDictionaryList.get(1);

                    BigDecimal value = new BigDecimal(item.getValue());
                    BigDecimal bili = new BigDecimal("0.01");
                    BigDecimal fencheng = value.multiply(bili);
                    BigDecimal commission = insurancePolicy.getCommission();
                    BigDecimal finalFee = commission.multiply(fencheng);

                    insurancePolicy.setAgentFee(finalFee);


                }
            }
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
