package com.jpsoft.picc.modules.base.service.impl;

import java.security.Policy;
import java.text.SimpleDateFormat;
import java.util.*;
import javax.annotation.Resource;

import cn.hutool.core.date.DateTime;
import cn.hutool.json.JSONObject;
import com.jpsoft.picc.modules.base.dao.CompanyDAO;
import com.jpsoft.picc.modules.base.dao.CompanyUserDAO;
import com.jpsoft.picc.modules.base.dao.MessageDAO;
import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.base.entity.CompanyUser;
import com.jpsoft.picc.modules.base.entity.Message;
import com.jpsoft.picc.modules.business.dao.InsuranceApplicationDAO;
import com.jpsoft.picc.modules.business.dao.InsurancePolicyDAO;
import com.jpsoft.picc.modules.business.entity.InsuranceApplication;
import com.jpsoft.picc.modules.business.entity.InsurancePolicy;
import com.jpsoft.picc.modules.common.config.WeixinConfig;
import com.jpsoft.picc.modules.common.constant.PolicyStatus;
import com.jpsoft.picc.modules.common.utils.JsonUtil;
import com.jpsoft.picc.modules.common.utils.VelocityHelper;
import com.jpsoft.picc.modules.common.utils.WeixinUtil;
import com.jpsoft.picc.modules.sys.dao.UserDAO;
import com.jpsoft.picc.modules.sys.entity.User;
import com.jpsoft.picc.modules.sys.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import com.jpsoft.picc.modules.base.dao.TemplateMessageDAO;
import com.jpsoft.picc.modules.base.entity.TemplateMessage;
import com.jpsoft.picc.modules.base.service.TemplateMessageService;
import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.github.pagehelper.PageHelper;

@Slf4j
@Transactional
@Component(value="templateMessageService")
public class TemplateMessageServiceImpl implements TemplateMessageService {
	@Resource(name="templateMessageDAO")
	private TemplateMessageDAO templateMessageDAO;

	@Resource(name="messageDAO")
	private MessageDAO messageDAO;

	@Autowired
	private InsurancePolicyDAO insurancePolicyDAO;

	@Autowired
	private InsuranceApplicationDAO insuranceApplicationDAO;

	@Autowired
	private WeixinConfig weixinConfig;

	@Autowired
	private CompanyDAO companyDAO;

	@Autowired
	private CompanyUserDAO companyUserDAO;

	@Autowired
	private UserDAO userDAO;

	@Override
	public TemplateMessage get(String id) {
		// TODO Auto-generated method stub
		return templateMessageDAO.get(id);
	}

	@Override
	public int insert(TemplateMessage model) {
		// TODO Auto-generated method stub
		//model.setId(UUID.randomUUID().toString());
		
		return templateMessageDAO.insert(model);
	}

	@Override
	public int update(TemplateMessage model) {
		// TODO Auto-generated method stub
		return templateMessageDAO.update(model);		
	}

	@Override
	public int delete(String id) {
		// TODO Auto-generated method stub
		return templateMessageDAO.delete(id);
	}

	@Override
	public boolean exist(String id) {
		// TODO Auto-generated method stub
		int count = templateMessageDAO.exist(id);
		
		return count > 0 ? true : false;
	}
	
	@Override
	public List<TemplateMessage> list() {
		// TODO Auto-generated method stub
		return templateMessageDAO.list();
	}
		
	@Override
	public Page<TemplateMessage> pageSearch(Map<String, Object> searchParams, int pageNumber, int pageSize,List<Sort> sortList) {
        Page<TemplateMessage> page = PageHelper.startPage(pageNumber,pageSize).doSelectPage(()->{
            templateMessageDAO.search(searchParams,sortList);
        });
        
        return page;
	}

	@Override
	public boolean sendTemplateMessage(String policyId,PolicyStatus srcPolicyStatus, PolicyStatus destPolicyStatus) {
		boolean result = false;

		try {
			log.warn(srcPolicyStatus.getText() + "->" + destPolicyStatus.getText());

			InsurancePolicy insurancePolicy = insurancePolicyDAO.get(policyId);
			InsuranceApplication insuranceApplication = insuranceApplicationDAO.get(insurancePolicy.getLatestApplicationId());
			Company company = companyDAO.get(insurancePolicy.getCompanyId());
			CompanyUser companyUser = companyUserDAO.findByCompanyId(company.getId());

			List<TemplateMessage> templateMessageList = templateMessageDAO.findByPolicyStatus(destPolicyStatus.getValue());

			for (TemplateMessage templateMessage : templateMessageList) {
				if (templateMessage==null){
					throw new Exception(destPolicyStatus.getText() + "对应模板消息不存在!");
				}

				List<Map> targetList = new ArrayList<>();

				if (templateMessage.getTarget().equals("USER")) {
					//发送给用户
					Map<String, Object> userMap = new HashMap<>();
					userMap.put("userId", companyUser.getId());
					userMap.put("openId", companyUser.getOpenId());
					userMap.put("target", templateMessage.getTarget());

					targetList.add(userMap);
				} else if (templateMessage.getTarget().equals("JZRCCS") || templateMessage.getTarget().equals("PICC")) {
					List<User> userList = userDAO.findByRoleName(templateMessage.getTarget());

					for (User user : userList) {
						Map<String, Object> userMap = new HashMap<>();
						userMap.put("userId", user.getId());
						userMap.put("openId", user.getOpenId());
						userMap.put("target", templateMessage.getTarget());

						targetList.add(userMap);
					}
				}

				for (Map map : targetList) {
					String userId = (String) map.get("userId");
					String openId = (String) map.get("openId");
					String target = (String) map.get("target");

					//如果是下月投保，则流程跳转到待制单环节时，不需要通知用户缴费成功
					if (target.equals("USER") && destPolicyStatus.equals(PolicyStatus.PendingMakePolicy) && insurancePolicy.getNo()>1){
						continue;
					}

					sendMessage(company.getName(),userId, openId,templateMessage, insuranceApplication, insurancePolicy, companyUser,srcPolicyStatus, destPolicyStatus);
				}
			}

			result = true;
		}
		catch (Exception ex){
			log.error(ex.getMessage(),ex);
			result = false;
		}

		return result;
	}

	/**
	 *
	 * @param companyName 单位名称
	 * @param userId 接收消息用户编号
	 * @param openId 接收消息用户微信openId
	 * @param templateMessage 模板消息
	 * @param insuranceApplication 投保申请单
	 * @param insurancePolicy 投保单
	 * @param companyUser 企业用户
	 * @param srcPolicyStatus 审核前状态
	 * @param destPolicyStatus 审核后状态
	 */
	private void sendMessage(String companyName,String userId,String openId,
							 TemplateMessage templateMessage,
							 InsuranceApplication insuranceApplication,
							 InsurancePolicy insurancePolicy,
							 CompanyUser companyUser,
							 PolicyStatus srcPolicyStatus,PolicyStatus destPolicyStatus){
		String content = templateMessage.getContent();

		Map<String,Object> contentVars = new HashMap<>();
		contentVars.put("userName", companyUser.getUserName());
		contentVars.put("companyName", companyName);

		//todo 如果是退回显示是从哪个步骤退回
		if (destPolicyStatus == PolicyStatus.Back){
			if (srcPolicyStatus == PolicyStatus.PendingTrial) {
				contentVars.put("approver", "人才超市");
			}
			else if(srcPolicyStatus == PolicyStatus.PendingRetrial){
				contentVars.put("approver","中国人保");
			}
		}

		content = VelocityHelper.format(content,contentVars);

		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

		Map<String,Object> jsonVars = new HashMap<>();

		jsonVars.put("content",content);
		jsonVars.put("time", DateTime.now().toString("yyyy-MM-dd HH:mm"));
		jsonVars.put("insuredName",insuranceApplication.getInsuredName());
		jsonVars.put("insuranceName",insuranceApplication.getDefinitionName());
		jsonVars.put("duration",sdf.format(insurancePolicy.getStartTime()) + "至" + sdf.format(insurancePolicy.getEndTime()));

		String json = VelocityHelper.format(templateMessage.getJson(),jsonVars);
		JSONObject jsonObject = new JSONObject(json);

		if (StringUtils.isNotEmpty(openId)) {
			//发送微信模板消息
			new Thread(() -> WeixinUtil.sendTemplateMessage(weixinConfig,
					openId, templateMessage.getCode(), jsonObject)
			).start();
		}

		//写站内消息
		Message message = new Message();
		message.setId(UUID.randomUUID().toString());
		message.setTitle(templateMessage.getTitle());

		if(destPolicyStatus == PolicyStatus.Back){
			//退回时显示退回步骤
			message.setTitle(templateMessage.getTitle() + "-" + srcPolicyStatus.getText());
		}

		//todo 20-人才超市核准,30-中国人保审核,50-待制单,60-待出单，调整发消息的内容
		if (destPolicyStatus.getValue()==20
		 || destPolicyStatus.getValue()==30
		 || destPolicyStatus.getValue()==50
		 || destPolicyStatus.getValue()==60) {
			message.setContent("【" + destPolicyStatus.getText() +"】" + content);
		}
		else{
			message.setContent(content);
		}

		message.setRecipientId(userId);
		message.setStatus(false);
		message.setDelFlag(false);
		message.setCreateTime(new Date());

		messageDAO.insert(message);
	}
}