package com.jpsoft.picc.modules.auth.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinition;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionService;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Api(description="险种信息")
@RestController
@RequestMapping("/auth/insuranceDefinition")
@Slf4j
public class InsuranceDefinitionController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsuranceDefinitionService insuranceDefinitionService;

    @ApiOperation(value="所有险种列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    public MessageResult<List<InsuranceDefinition>> list(){
        MessageResult<List<InsuranceDefinition>> msgResult = new MessageResult<>();

        List<InsuranceDefinition> list = insuranceDefinitionService.list();

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }

    @ApiOperation(value="查看详情")
    @RequestMapping(value = "detail",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form",dataType = "String"),
    })
    public MessageResult<InsuranceDefinition> detail(String id){
        MessageResult<InsuranceDefinition> msgResult = new MessageResult<>();

        try {
            InsuranceDefinition insuranceDefinition = insuranceDefinitionService.get(id);

            if(insuranceDefinition != null) {
                msgResult.setData(insuranceDefinition);
                msgResult.setResult(true);
            }else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
