package com.jpsoft.picc.modules.auth.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.auth.dto.AcceptInsuranceDTO;
import com.jpsoft.picc.modules.auth.dto.InsuranceApplicationDTO;
import com.jpsoft.picc.modules.auth.dto.InsuranceJobsDTO;
import com.jpsoft.picc.modules.auth.dto.InsurancePolicyRecordDTO;
import com.jpsoft.picc.modules.base.entity.*;
import com.jpsoft.picc.modules.base.service.*;
import com.jpsoft.picc.modules.business.entity.*;
import com.jpsoft.picc.modules.business.service.*;
import com.jpsoft.picc.modules.common.config.OSSConfig;
import com.jpsoft.picc.modules.common.constant.PolicyStatus;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.utils.OSSUtil;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.utils.VelocityHelper;
import com.jpsoft.picc.modules.sys.entity.User;
import com.jpsoft.picc.modules.sys.service.DataDictionaryService;
import com.jpsoft.picc.modules.sys.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.jasig.cas.client.authentication.AttributePrincipal;
import org.joda.time.DateTime;
import org.joda.time.Interval;
import org.joda.time.Months;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

@Slf4j
@Api(description="下月投保单")
@RestController
@RequestMapping("/auth/insurancePolicy")
public class InsurancePolicyController {
    @Autowired
    private OSSConfig ossConfig;
    @Autowired
    private InsurancePolicyService insurancePolicyService;

    @Autowired
    private InsurancePolicyRecordService insurancePolicyRecordService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private CompanyUserService companyUserService;

    @Autowired
    private InsuranceApplicationService insuranceApplicationService;

    @Autowired
    private InsurancePolicyMemberService insurancePolicyMemberService;

    @Autowired
    private ApplicationPolicyService applicationPolicyService;

    @Autowired
    private InsuranceJobsService insuranceJobsService;

    @Autowired
    private InsuranceDefinitionLimitService insuranceDefinitionLimitService;

    @Autowired
    private InsuranceDefinitionService insuranceDefinitionService;

    @Autowired
    private TransactionRecordService transactionRecordService;

    @Autowired
    private JobsService jobsService;

    @Autowired
    private UserService userService;

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private TemplateMessageService templateMessageService;

    @ResponseBody
    @ApiOperation(value="附件列表")
    @RequestMapping(value = "attachmentList",method = RequestMethod.GET)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "policyId",value = "投保单编号", required = true, paramType = "query"),
    })
    public MessageResult<List> attachmentList(String policyId,HttpServletRequest request){
        MessageResult<List> msgResult = new MessageResult<>();

        try {
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            String path = request.getContextPath();
            String basePath = request.getScheme()+"://"+request.getServerName()+":"+request.getServerPort()+path+"/";

            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
            Company company = companyService.get(companyUser.getCompanyId());

            List<Map> mapList = new ArrayList<>();

            Map<String,String> map1 = new HashMap<>();

            String url = company.getUsccFiles();

            if(StringUtils.isNotEmpty(url)) {
                map1.put("type", "营业执照");
                map1.put("title", "营业执照" + url.substring(url.lastIndexOf(".")));
                map1.put("url", url);

                mapList.add(map1);
            }

            Map<String,String> map2 = new HashMap<>();
            map2.put("type","身份证");
            map2.put("title","身份证.zip");
            map2.put("url",basePath + "auth/attachment/downloadIdCardZip?policyId=" + policyId);

            mapList.add(map2);

            msgResult.setData(mapList);
            msgResult.setResult(true);
        }
        catch (Exception ex){
            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }

    @ApiOperation(value="查看退回理由")
    @RequestMapping(value = "backReason",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "policyId",value = "每月投保单ID", required = true, paramType = "form",dataType = "String")
    })
    public MessageResult<InsurancePolicyRecordDTO> backReason(@RequestParam(value="policyId",defaultValue="") String policyId){
        MessageResult<InsurancePolicyRecordDTO> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();
        searchParams.put("policyId",policyId);

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        Page<InsurancePolicyRecord> page = insurancePolicyRecordService.pageSearch(searchParams,1,100,sortList);
        if(page.size() > 0){
            InsurancePolicyRecordDTO insurancePolicyRecordDTO = new InsurancePolicyRecordDTO();
            PojoUtils.map(page.getResult().get(0), insurancePolicyRecordDTO);

//          Company company = companyService.get(insurancePolicyRecordDTO.getCreateBy());
//          insurancePolicyRecordDTO.setCreateByName(company.getName());
            User user = userService.get(insurancePolicyRecordDTO.getCreateBy());

            if(user!=null){
                insurancePolicyRecordDTO.setCreateByName(user.getRealName());
            }

            msgResult.setResult(true);
            msgResult.setData(insurancePolicyRecordDTO);
        }else {
            msgResult.setResult(false);
            msgResult.setMessage("没有相关退回理由。");
        }

        return msgResult;
    }

    @PostMapping(value="historyList")
    @ApiOperation(value = "根据投保单号查询每月投保单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "applicationId",value = "参保单编号", required = true, paramType = "form"),
    })
    public MessageResult<Map> historyList(String applicationId,
                                          @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
                                          @RequestParam(value="pageSize",defaultValue="20") int pageSize){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            //todo
            Page<InsurancePolicy> page = applicationPolicyService.findPolicyByApplicationId(applicationId,pageIndex,pageSize);

            Map dataMap = PojoUtils.pageWrapper(page);

            List<Map> mapList = new ArrayList<>();

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

            Date now = new Date();

            for (InsurancePolicy policy : page.getResult()) {
                Map<String,Object> map = new HashMap<>();

                map.put("applicationId",applicationId);
                map.put("policyId",policy.getId());
                map.put("startTime",sdf.format(policy.getStartTime()));
                map.put("endTime",sdf.format(policy.getEndTime()));
                map.put("definitionName",policy.getDefinitionName());
                //电子投保单下载地址
                map.put("insurancePolicyFile", policy.getInsurancePolicyFile());
                map.put("electronicInvoiceFile", policy.getElectronicInvoiceFile());
                map.put("policyNo",policy.getPolicyNo());
                map.put("createTime", sdf2.format(policy.getCreateTime()));

                if (policy.getFinishTime()!=null) {
                    map.put("finishTime", sdf2.format(policy.getFinishTime()));
                }

                try {
                    map.put("status", PolicyStatus.valueOf(Integer.valueOf(policy.getStatus())).getText());
                }
                catch (Exception ex){
                    log.error(ex.getMessage(),ex);
                }
//                if (now.before(policy.getStartTime())){
//                    map.put("status","未生效");
//                }
//                else if(now.after(policy.getEndTime())){
//                    map.put("status","已过期");
//                }
//                else{
//                    map.put("status","保障中");
//                }

                mapList.add(map);
            }

            dataMap.put("data",mapList);

            messageResult.setData(dataMap);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @ApiOperation(value="下月投保单分页查询")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "status",value = "状态:待调整10/已退回15/待初审20/待复审30/待制单50/待出单60/已出单70", required = true, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "definitionId",value = "险种编号", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "startTime",value = "起始时间", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "endTime",value = "结束时间", required = false, paramType = "form",dataType = "String"),
    })
    public MessageResult<Map> pageList(
            String status,String definitionId,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date startTime,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date endTime,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            HttpServletRequest request){
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
            Company company = null;

            if (companyUser!=null && StringUtils.isNotEmpty(companyUser.getCompanyId())) {
                company = companyService.get(companyUser.getCompanyId());
            }

            if (company==null){
                throw new Exception("请先填写企业信息!");
            }

            Map<String, Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "desc"));

            if (StringUtils.isNotEmpty(company.getId())) {
                searchParams.put("companyId", company.getId());
            }

            if (StringUtils.isNotEmpty(definitionId)) {
                searchParams.put("definitionId", definitionId);
            }

            if (StringUtils.isNotEmpty(status)) {
                searchParams.put("status", status);
            }

            if (startTime != null) {
                searchParams.put("startTime", startTime);
            }

            if (endTime != null) {
                searchParams.put("endTime", endTime);
            }

            Page<InsurancePolicy> page = insurancePolicyService.nextMonthPageList(searchParams, pageIndex, pageSize,sortList);

            msgResult.setData(PojoUtils.pageWrapper(page));
            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="显示下月参保人员列表")
    @RequestMapping(value = "memberPageList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "policyId",value = "每月投保单编号", required = true, paramType = "form",dataType = "String")
    })
    public MessageResult<Map> memberPageList(
            @RequestParam(value="policyId",defaultValue="") String policyId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize){
        MessageResult<Map> msgResult = new MessageResult<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","asc"));
        Map<String,Object> searchParams = new HashMap<>();
        searchParams.put("policyId",policyId);

        Page<InsurancePolicyMember> page = insurancePolicyMemberService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="提交下月花名册")
    @RequestMapping(value = "submitRoster",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "policyId",value = "每月投保单编号", required = true, paramType = "form",dataType = "String")
    })
    public MessageResult<InsuranceApplication> submitRoster(String policyId,HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        try {
            InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);

            //todo 查询是否有待合并的投保单存在，如果有说明新增人员未完成，不能提交
            List<InsuranceApplication> insuranceApplicationList = insuranceApplicationService.findByCombinePolicyId(insurancePolicy.getId());

            if (insuranceApplicationList.size()>0){
                throw new Exception("有新增投保人员流程未完成！");
            }

            List<InsurancePolicyMember> insurancePolicyMemberList = insurancePolicyMemberService.findByPolicyId(insurancePolicy.getId());

            boolean isUnChecked = false;

            for (InsurancePolicyMember insurancePolicyMember : insurancePolicyMemberList) {
                String memberStatus = insurancePolicyMember.getStatus();
                //花名册人员是否为已复核状态
                if (!("2").equals(memberStatus)) {
                    isUnChecked = true;
                    break;
                }
            }

            PolicyStatus policyStatus;

            if (isUnChecked) {
                //有未复核人员，则状态为待初审
                policyStatus = PolicyStatus.PendingTrial;
                insurancePolicy.setStatus(policyStatus.getValue() + "");
            } else {
                //所有人员都已复核吗，则状态为待制单
                policyStatus = PolicyStatus.PendingMakePolicy;
                insurancePolicy.setStatus(policyStatus.getValue() + "");
            }

            insurancePolicy.setUpdateBy(principal.getName());
            insurancePolicy.setUpdateTime(new Date());

            insurancePolicyService.update(insurancePolicy);

            templateMessageService.sendTemplateMessage(policyId,PolicyStatus.Draft,policyStatus);

            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="获取投保信息")
    @RequestMapping(value = "insureDetail",method = RequestMethod.GET)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "policyId",value = "每月投保编号", required = true, paramType = "query",dataType = "String")
    })
    public MessageResult<InsuranceApplicationDTO> insureDetail(String policyId){
        MessageResult<InsuranceApplicationDTO> msgResult = new MessageResult<>();

        InsuranceApplicationDTO insuranceApplicationDTO = new InsuranceApplicationDTO();

        InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);

        try {
            InsuranceApplication insuranceApplication = null;

            List<InsuranceApplication> appList = applicationPolicyService.findApplicationByPolicyId(policyId);

            if(appList.size()>0){
                insuranceApplication = appList.get(0);
            }

            if (insuranceApplication != null) {
                PojoUtils.map(insuranceApplication, insuranceApplicationDTO);
                insuranceApplicationDTO.setApplicationId(insuranceApplication.getId());

                //营业性质
//                if (StringUtils.isNotEmpty(insuranceApplication.getBusinessNature())){
//                    String businessNatureName = dataDictionaryService.findNameByCatalogNameAndValue("营业性质",insuranceApplication.getBusinessNature());
//                    insuranceApplicationDTO.setBusinessNatureN(businessNatureName);
//                }

                DateTime endTime = new DateTime(insuranceApplication.getEndTime());
                DateTime startTime = new DateTime(insuranceApplication.getStartTime());

                insuranceApplicationDTO.setMonthNumber(Months.monthsBetween(startTime, endTime).getMonths());
            }

            //设置参保人数为当前月投保单人数
            insuranceApplicationDTO.setInsuredNumber(insurancePolicy.getNumber());

            msgResult.setData(insuranceApplicationDTO);

            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取承保信息")
    @RequestMapping(value = "acceptDetail",method = RequestMethod.GET)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "policyId",value = "每月投保单ID", required = true,paramType = "query",dataType = "String"),
    })
    public MessageResult<AcceptInsuranceDTO> acceptDetail(String policyId){
        MessageResult<AcceptInsuranceDTO> msgResult = new MessageResult<>();
        AcceptInsuranceDTO acceptInsuranceDTO = new AcceptInsuranceDTO();

        try {
            //每月投保单
            InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);

            //险种
            InsuranceDefinition insuranceDefinition = insuranceDefinitionService.get(insurancePolicy.getDefinitionId());

            //险种名称
            acceptInsuranceDTO.setInsuranceDefName(insuranceDefinition.getName());

            //险种关联限额
            List<InsuranceDefinitionLimit> insuranceDefinitionLimits = insuranceDefinitionLimitService.findByDefinitionId(insuranceDefinition.getId());

            Map<String,Object> varsMap = new HashMap<>();
            varsMap.put("num",insurancePolicy.getNumber());

            for(InsuranceDefinitionLimit  insuranceDefinitionLimit : insuranceDefinitionLimits){
                //限额这里可能是公式，需要带入人数
                String limit = VelocityHelper.format(insuranceDefinitionLimit.getLimit(),varsMap);
                insuranceDefinitionLimit.setLimit(limit);

                String unit = VelocityHelper.format(insuranceDefinitionLimit.getUnit(),varsMap);
                insuranceDefinitionLimit.setUnit(unit);
            }

            acceptInsuranceDTO.setInsuranceDefLimitList(insuranceDefinitionLimits);

            //关联岗位
            List<InsuranceJobs> InsuranceJobsList = insuranceJobsService.findByDefinitionId(insuranceDefinition.getId());

            //todo 查看每月投保记录时显示当前月
            DateTime endTime = new DateTime(insurancePolicy.getEndTime());
            acceptInsuranceDTO.setEndTime(insurancePolicy.getEndTime());

            DateTime startTime = new DateTime(insurancePolicy.getStartTime());
            acceptInsuranceDTO.setStartTime(insurancePolicy.getStartTime());

            //投保总月份
//            int months = Months.monthsBetween(startTime, endTime).getMonths();
            int months = 1;

            BigDecimal totalAmount = new BigDecimal(0);
            int totalNumber = 0;

            List<InsuranceJobsDTO> insuranceJobsList = new ArrayList<>();

            //查询不同岗位，参保费用
            for(InsuranceJobs insuranceJobs : InsuranceJobsList){
                Jobs jobs = jobsService.get(insuranceJobs.getJobsId());

                InsuranceJobsDTO insuranceJobsDTO = new InsuranceJobsDTO();

                insuranceJobsDTO.setJobs(jobs);

                long memberCount = insurancePolicyMemberService.countByPolicyIdAndJobsId(policyId,insuranceJobs.getJobsId());

                insuranceJobsDTO.setCharges(insuranceJobs.getCharges());
                insuranceJobsDTO.setChargesY(insuranceJobs.getCharges().multiply(new BigDecimal(12)));
                insuranceJobsDTO.setNumber(Long.valueOf(memberCount).intValue());

                insuranceJobsDTO.setMoney(insuranceJobsDTO.getCharges()
                        .multiply(new BigDecimal(months))
                        .multiply(new BigDecimal(insuranceJobsDTO.getNumber())));

                totalAmount = totalAmount.add(insuranceJobsDTO.getMoney());
                totalNumber += memberCount;

                insuranceJobsList.add(insuranceJobsDTO);
            }

            acceptInsuranceDTO.setInsuranceJobsList(insuranceJobsList);
            acceptInsuranceDTO.setTotalAmount(totalAmount);

            msgResult.setData(acceptInsuranceDTO);
            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @PostMapping(value="changeStatus")
    @ApiOperation(value = "修改投保单状态(测试用)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "policyId",value = "投保单编号", required = true, paramType = "form"),
            @ApiImplicitParam(name = "status",value = "状态:草稿10/已退回15/待初审20/待复审30/待缴费40/待制单50/待出单60/已出单70/已过期100", required = true, paramType = "form"),
    })
    public MessageResult<String> changeStatus(String policyId,String status,HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

        MessageResult<String> messageResult = new MessageResult<>();

        try {
            //todo
            InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);
            insurancePolicy.setStatus(status);
            insurancePolicy.setUpdateBy(principal.getName());
            insurancePolicy.setUpdateTime(new Date());

            insurancePolicyService.update(insurancePolicy);

            if (insurancePolicy.getNo().equals(1)){
                InsuranceApplication insuranceApplication = insuranceApplicationService.get(insurancePolicy.getLatestApplicationId());
                insuranceApplication.setStatus(status);
                insuranceApplication.setUpdateBy(principal.getName());
                insurancePolicy.setUpdateTime(new Date());

                insuranceApplicationService.update(insuranceApplication);
            }

            messageResult.setResult(true);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @ApiOperation(value="下月投保单状态分类数量")
    @RequestMapping(value = "statusCount",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "definitionId",value = "险种编号", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "startTime",value = "起始时间", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "endTime",value = "结束时间", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "status",value = "投保单状态", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<Map> statusCount(String definitionId,
                                       @DateTimeFormat(pattern = "yyyy-MM-dd") Date startTime,
                                       @DateTimeFormat(pattern = "yyyy-MM-dd") Date endTime,
                                       String status,
                                       HttpServletRequest request){
        MessageResult<Map> msgResult = new MessageResult<>();
        Map<String, Object> dataMap = new HashMap<>();

        try {
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
            Company company = null;

            if(companyUser!=null && StringUtils.isNotEmpty(companyUser.getCompanyId())) {
                company = companyService.get(companyUser.getCompanyId());
            }

            Map<String, Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "desc"));

            if (company!=null && StringUtils.isNotEmpty(company.getId())) {
                searchParams.put("companyId", company.getId());
            }
            else{
                searchParams.put("companyId", "");
            }

            if (StringUtils.isNotEmpty(definitionId)) {
                searchParams.put("definitionId", definitionId);
            }

            if (startTime != null) {
                searchParams.put("startTime", startTime);
            }

            if (endTime != null) {
                searchParams.put("endTime", endTime);
            }

            for(PolicyStatus policyStatus : PolicyStatus.values()) {
                boolean processed = false;

                if (StringUtils.isEmpty(status)){
                    processed = true;
                }
                else if(status.equals(String.valueOf(policyStatus.getValue()))){
                    processed = true;
                }

                if (processed) {
                    searchParams.put("status", policyStatus.getValue() + "");
                    Page<InsurancePolicy> page = insurancePolicyService.nextMonthPageList(searchParams, 1, 1000, sortList);

                    dataMap.put(policyStatus.getValue() + "", page.getTotal());
                }
            }

            msgResult.setData(dataMap);
            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @PostMapping(value="addMemberOfNextMonthPolicy")
    @ApiOperation(value = "下月投保单中新增人员")
    @Transactional(rollbackFor = Exception.class)
    @ApiImplicitParams({
        @ApiImplicitParam(name = "policyId",value = "下月投保单", required = true, paramType = "form")
    })
    public MessageResult<Map> addMemberOfNextMonthPolicy(String policyId,HttpServletRequest request){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());

            //todo 新增一份投保单用于缴费,截止月份相同，起始月份为当前投保单月份，流程只需要经过初审、复审、缴费后就合并。
            InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);

            InsuranceApplication insuranceApplication = insuranceApplicationService.get(insurancePolicy.getLatestApplicationId());

            //todo 判断当前相同企业、相同险种、生效期限相同的投保单是否存在
//            List<InsuranceApplication> destList = insuranceApplicationService.findByDuration(insuranceApplication.getCompanyId(),
//                                                        insuranceApplication.getDefinitionId(),
//                                                        insurancePolicy.getStartTime(),
//                                                        insuranceApplication.getEndTime(),
//                                                        new String[]{PolicyStatus.Draft.getValue() + "",PolicyStatus.Back.getValue() + ""});
            List<InsuranceApplication> destList = insuranceApplicationService.findByCombinePolicyId(policyId);

            InsuranceApplication destApplication = null;

            if(destList.size()>0) {
                destApplication = destList.get(0);
            }

            if(destApplication==null) {
                destApplication = insuranceApplication;
                destApplication.setId(UUID.randomUUID().toString());

                //todo 新增参保申请单号与原参保申请单一致
                destApplication.setPolicyNo(insuranceApplication.getPolicyNo());
                destApplication.setCreateTime(new Date());
                destApplication.setCreateBy(principal.getName());
                destApplication.setInsuranceFee(new BigDecimal(0));
                destApplication.setInsuredNumber(0);
                destApplication.setStartTime(insurancePolicy.getStartTime()); //月份改为当前月份
                destApplication.setStatus(PolicyStatus.Draft.getValue() + "");
                destApplication.setDelFlag(false); //如果缴费后，将这个状态设为true
                destApplication.setCombinePolicyId(policyId); //设置要合并的每月投保单

                insuranceApplicationService.insert(destApplication);

                //todo 新增一份每月投保单
                InsurancePolicy newPolicy = new InsurancePolicy();
                newPolicy.setId(UUID.randomUUID().toString());
                newPolicy.setLatestApplicationId(destApplication.getId());
                newPolicy.setPolicyNo(insurancePolicy.getPolicyNo()); //由于是用来合并，保持与原投保单一致

                newPolicy.setDelFlag(false);
                newPolicy.setCreateBy(principal.getName());
                newPolicy.setCreateTime(new Date());
                newPolicy.setNo(1);

                int affectCount = insurancePolicyService.insert(newPolicy);

                if (affectCount > 0) {
                    //todo 关联投保申请单月与每月投保单
                    ApplicationPolicy applicationPolicy = new ApplicationPolicy();
                    applicationPolicy.setId(UUID.randomUUID().toString());

                    applicationPolicy.setApplicationId(destApplication.getId());
                    applicationPolicy.setPolicyId(newPolicy.getId());

                    applicationPolicyService.insert(applicationPolicy);
                }

                Map<String,Object> dataMap = new HashMap<>();

                dataMap.put("applicationId",destApplication.getId());
                dataMap.put("policyId",newPolicy.getId());
                dataMap.put("companyId",destApplication.getCompanyId());

                messageResult.setData(dataMap);
            }
            else{
                InsurancePolicy firstPolicy = applicationPolicyService.findFirstPolicyByApplicationId(destApplication.getId());

                Map<String,Object> dataMap = new HashMap<>();

                dataMap.put("applicationId",destApplication.getId());
                dataMap.put("policyId",firstPolicy.getId());
                dataMap.put("companyId",destApplication.getCompanyId());

                messageResult.setData(dataMap);
            }

            messageResult.setResult(true);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());

            //在实际开发中，我们往往需要在方法中进行异常的捕获，从而对异常进行判断，为客户端返回提示信息。
            //但是此时由于异常的被捕获，导致事务的回滚没有被触发，导致事务的失败。
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }

        return messageResult;
    }

    @ApiOperation(value="我的保单-分页列表")
    @RequestMapping(value = "pageListMyPolicy",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "status",value = "状态:已出单70/已过期100", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "definitionId",value = "险种编号", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "startTime",value = "创建起始时间", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "endTime",value = "创建结束时间", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "startFinishTime",value = "出单起始时间", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "endFinishTime",value = "出单结束时间", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<Map> pageListMyPolicy(
            String status,String definitionId,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date startTime,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date endTime,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date startFinishTime,
            @DateTimeFormat(pattern = "yyyy-MM-dd") Date endFinishTime,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            HttpServletRequest request){
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
            Company company = null;

            if (companyUser!=null && StringUtils.isNotEmpty(companyUser.getCompanyId())) {
                company = companyService.get(companyUser.getCompanyId());
            }

            if (company==null){
                throw new Exception("请先填写企业信息!");
            }

            Map<String, Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "desc"));

            if (StringUtils.isNotEmpty(company.getId())) {
                searchParams.put("companyId", company.getId());
            }

            if (StringUtils.isNotEmpty(definitionId)) {
                searchParams.put("definitionId", definitionId);
            }

            if (StringUtils.isNotEmpty(status)) {
                searchParams.put("status", status);
            }else{
                searchParams.put("myStatus", "true");
            }

            if (startTime != null) {
                searchParams.put("startTime", startTime);
            }

            if (endTime != null) {
                searchParams.put("endTime", endTime);
            }

            if (startFinishTime != null) {
                searchParams.put("startFinishTime", startFinishTime);
            }

            if (endFinishTime != null) {
                searchParams.put("endFinishTime", endFinishTime);
            }

            Page<InsurancePolicy> page = insurancePolicyService.myPageSearch(searchParams, pageIndex, pageSize,sortList);

            msgResult.setData(PojoUtils.pageWrapper(page));
            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="导出")
    @PostMapping("exportXls")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "status",value = "状态:已出单70/已过期100", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "definitionId",value = "险种编号", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "startTime",value = "创建起始时间", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "endTime",value = "创建结束时间", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "startFinishTime",value = "出单起始时间", required = false, paramType = "form",dataType = "String"),
            @ApiImplicitParam(name = "endFinishTime",value = "出单结束时间", required = false, paramType = "form",dataType = "String")
    })
    public MessageResult<String> exportXls(String status,String definitionId,
                                           @DateTimeFormat(pattern = "yyyy-MM-dd") Date startTime,
                                           @DateTimeFormat(pattern = "yyyy-MM-dd") Date endTime,
                                           @DateTimeFormat(pattern = "yyyy-MM-dd") Date startFinishTime,
                                           @DateTimeFormat(pattern = "yyyy-MM-dd") Date endFinishTime,
                                           HttpServletRequest request){
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            HSSFWorkbook workbook = new HSSFWorkbook();
            HSSFSheet sheet = workbook.createSheet();
            //表头
            HSSFRow rowTitle = sheet.createRow(0);
            HSSFCell cellTitle1 = rowTitle.createCell(0);
            cellTitle1.setCellValue("序号");
            HSSFCell cellTitle2 = rowTitle.createCell(1);
            cellTitle2.setCellValue("投保单号");
            HSSFCell cellTitle3 = rowTitle.createCell(2);
            cellTitle3.setCellValue("投保险种");
            HSSFCell cellTitle4 = rowTitle.createCell(3);
            cellTitle4.setCellValue("投保人数");
            HSSFCell cellTitle5 = rowTitle.createCell(4);
            cellTitle5.setCellValue("投保期间");
            HSSFCell cellTitle6 = rowTitle.createCell(5);
            cellTitle6.setCellValue("创建时间");
            HSSFCell cellTitle7 = rowTitle.createCell(6);
            cellTitle7.setCellValue("出单时间");
            HSSFCell cellTitle8 = rowTitle.createCell(7);
            cellTitle8.setCellValue("保单金额");
            HSSFCell cellTitle9 = rowTitle.createCell(8);
            cellTitle9.setCellValue("保单状态");

            //内容-获取数据
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
            Company company = null;

            if (companyUser!=null && StringUtils.isNotEmpty(companyUser.getCompanyId())) {
                company = companyService.get(companyUser.getCompanyId());
            }

            if (company==null){
                throw new Exception("请先填写企业信息!");
            }

            Map<String, Object> searchParams = new HashMap<>();

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "desc"));

            if (StringUtils.isNotEmpty(company.getId())) {
                searchParams.put("companyId", company.getId());
            }

            if (StringUtils.isNotEmpty(definitionId)) {
                searchParams.put("definitionId", definitionId);
            }

            if (StringUtils.isNotEmpty(status)) {
                searchParams.put("status", status);
            }else{
                searchParams.put("myStatus", "true");
            }

            if (startTime != null) {
                searchParams.put("startTime", startTime);
            }

            if (endTime != null) {
                searchParams.put("endTime", endTime);
            }

            if (startFinishTime != null) {
                searchParams.put("startFinishTime", startFinishTime);
            }

            if (endFinishTime != null) {
                searchParams.put("endFinishTime", endFinishTime);
            }

            Page<InsurancePolicy> page = insurancePolicyService.myPageSearch(searchParams, 1, 1000,sortList);

            //内容-写入数据
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            SimpleDateFormat sdfEx = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            for(int i=0;i<page.size();i++){
                InsurancePolicy insurancePolicy = page.get(i);

                HSSFRow rowContent = sheet.createRow(i + 1);
                HSSFCell cellContent1 = rowContent.createCell(0);
                cellContent1.setCellValue(i+1);
                HSSFCell cellContent2 = rowContent.createCell(1);
                if(insurancePolicy.getPolicyNo() != null) cellContent2.setCellValue(insurancePolicy.getPolicyNo());
                HSSFCell cellContent3 = rowContent.createCell(2);
                if(insurancePolicy.getDefinitionName() != null) cellContent3.setCellValue(insurancePolicy.getDefinitionName());
                HSSFCell cellContent4 = rowContent.createCell(3);
                if(insurancePolicy.getNumber() != null) cellContent4.setCellValue(insurancePolicy.getNumber());
                HSSFCell cellContent5 = rowContent.createCell(4);
                if(insurancePolicy.getStartTime() != null && insurancePolicy.getEndTime() != null) cellContent5.setCellValue(sdf.format(insurancePolicy.getStartTime()) + "至" + sdf.format(insurancePolicy.getEndTime()));
                HSSFCell cellContent6 = rowContent.createCell(5);
                if(insurancePolicy.getCreateTime() != null) cellContent6.setCellValue(sdfEx.format(insurancePolicy.getCreateTime()));
                HSSFCell cellContent7 = rowContent.createCell(6);
                if(insurancePolicy.getFinishTime() != null) cellContent7.setCellValue(sdfEx.format(insurancePolicy.getFinishTime()));
                HSSFCell cellContent8 = rowContent.createCell(7);
                if(insurancePolicy.getInsuranceFee() != null) cellContent8.setCellValue(insurancePolicy.getInsuranceFee().floatValue());
                HSSFCell cellContent9 = rowContent.createCell(8);
                if(insurancePolicy.getStatus() != null) cellContent9.setCellValue(insurancePolicy.getStatusN());
            }

            //todo 将wb保存到oss
            ByteArrayOutputStream output = new ByteArrayOutputStream();
            workbook.write(output);

            byte[] buffer = output.toByteArray();
            ByteArrayInputStream input = new ByteArrayInputStream(buffer);

            String downloadUrl = OSSUtil.upload(ossConfig,"import","我的保单.xls",input);

            //todo 返回导入失败报表下载链接

            msgResult.setData(downloadUrl);
            msgResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
