package com.jpsoft.picc.modules.pub.controller;

import com.jpsoft.picc.config.CasConfig;
import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.base.entity.CompanyUser;
import com.jpsoft.picc.modules.base.service.CompanyService;
import com.jpsoft.picc.modules.base.service.CompanyUserService;
import com.jpsoft.picc.modules.common.config.WeixinConfig;
import com.jpsoft.picc.modules.common.constant.WeixinEvent;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.utils.DES3;
import com.jpsoft.picc.modules.common.utils.WeixinUtil;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.DeleteMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.apache.commons.lang3.StringUtils;
import org.jasig.cas.client.authentication.AttributePrincipal;
import org.jasig.cas.client.util.AbstractCasFilter;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.lang.reflect.Executable;
import java.net.URLEncoder;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Slf4j
@CrossOrigin
@Controller
@Api(description = "用户登录、登出")
public class UserController {
    @Autowired
    private CasConfig casConfig;

    @Autowired
    private WeixinConfig weixinConfig;

    @Autowired
    private CompanyUserService companyUserService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private ValueOperations<String,Object> valueOperations;

    @ResponseBody
    @ApiOperation(value = "创建扫码登录二维码")
    @GetMapping(value="/qrcode/login/create")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "loginCode",value = "6位随机数", required = true, paramType = "query")
    })
    public MessageResult<String> createQrcodeLogin(@RequestParam(value = "loginCode") String loginCode){
        MessageResult<String> msgResult = new MessageResult<>();

        log.warn("收到登录请求码：" + loginCode);

        try {
            if (loginCode.length()>10){
                throw new Exception("登录请求码长度不能大于10!");
            }

            long expireSeconds = 3000; //5分钟
            String url = WeixinUtil.createQrcode(weixinConfig, WeixinEvent.PICC_ENT_SCAN_QRCODE_LOGIN + "," + loginCode, expireSeconds);

            if(StringUtils.isNotEmpty(url)){
                valueOperations.set(WeixinConfig.SCAN_QRCODE_LOGIN_PREFIX + loginCode,"0",expireSeconds, TimeUnit.SECONDS);

                msgResult.setData(url);
                msgResult.setResult(true);
            }
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }

    /**
     * 接收扫码登录回调
     * @param eventKey
     * @param openId
     * @return 返回值会在微信中显示
     */
    @ApiOperation(value = "接收扫码登录回调")
    @PostMapping(value="/qrcode/login/scan")
    @ResponseBody
    public String scanQrcodeLogin(String eventKey,String openId){
        log.warn(openId + "请求登录!");
        String result;

        CompanyUser companyUser = companyUserService.findByOpenId(openId);

        if(companyUser!=null) {
            String[] arr = eventKey.split(",");
            String randNum = arr[1];

            long expireSeconds = 3000; //5分钟

            valueOperations.set(WeixinConfig.SCAN_QRCODE_LOGIN_PREFIX + randNum, openId, expireSeconds, TimeUnit.SECONDS);
            result = "扫码登录成功!";
        }
        else{
            result = "当前用户未绑定微信!";
        }

        return result;
    }

    @ApiOperation(value = "扫码登录轮询")
    @GetMapping(value="/qrcode/login/loop")
    @ResponseBody
    public MessageResult<String> scanQrcodeLoginLoop(String loginCode){
        MessageResult<String> msgResult = new MessageResult<>();

        String openId = (String) valueOperations.get(WeixinConfig.SCAN_QRCODE_LOGIN_PREFIX + loginCode);

        if(StringUtils.isNotEmpty(openId) && openId.length()>1) {
            //n秒内可以用该openId登录
            valueOperations.set("LOGIN_" + openId, "1", 120,TimeUnit.SECONDS);

            msgResult.setData(openId);
        }
        else{
            //由于前端将result=false的message全部弹出显示，故这里改成设置data=""
            msgResult.setData("");
            msgResult.setMessage("未收到登录请求!");
        }

        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value = "获取微信绑定二维码")
    @GetMapping(value="/auth/qrcode/binding/create")
    @ResponseBody
    public MessageResult<String> createBindingQrcode(HttpServletRequest request){
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());

            String url = WeixinUtil.createQrcode(weixinConfig,
                    WeixinEvent.PICC_ENT_SCAN_QRCODE_BINDING + "," + companyUser.getId(), 3000);

            if(StringUtils.isNotEmpty(url)){
                msgResult.setData(url);
                msgResult.setResult(true);
            }
            else{
                throw new Exception("生成二维码失败!");
            }
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);
            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }

    /**
     * 接收微信绑定回调
     * @param eventKey
     * @param openId
     * @return 返回值会在微信中显示
     */
    @ApiOperation(value = "接收微信绑定回调")
    @PostMapping(value="/qrcode/binding/scan")
    @ResponseBody
    public String scanQrcodeBinding(String eventKey,String openId){
        String result = "";

        try {
            String[] arr = eventKey.split(",");

            CompanyUser companyUser = companyUserService.get(arr[1]);

            companyUser.setOpenId(openId);
            companyUser.setUpdateBy("weixin");
            companyUser.setUpdateTime(new Date());

            companyUserService.update(companyUser);

            result= "微信绑定成功!";
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            result = "微信绑定失败!" + ex.getMessage();
        }

        return result;
    }

    @PostMapping(value="/login")
    @ResponseBody
    @ApiOperation(value="登录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userName",value = "用户名", required = false, paramType = "form"),
            @ApiImplicitParam(name = "password",value = "密码", required = false, paramType = "form"),
            @ApiImplicitParam(name = "openId",value = "微信OpenID", required = false, paramType = "form"),
            @ApiImplicitParam(name = "service",value = "跳转地址", required = true, paramType = "form")
    })
    public MessageResult<Map> login(
            String userName,String password,String openId,
            String service,HttpSession session) throws Exception{
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            DES3 des3 = new DES3();

            //如果请求过微信登录则，当前openId在redis就值
            if (StringUtils.isNotEmpty(openId) && valueOperations.get("LOGIN_" + openId)!=null) {
                CompanyUser companyUser = companyUserService.findByOpenId(openId);

                if(companyUser!=null) {
                    userName = companyUser.getUserName();
                    password = des3.decrypt(DES3.DEFAULT_KEY, companyUser.getPassword());
                }
                else{
                    throw new Exception("用户未绑定微信!");
                }
            }

            String casServerTicketUrl = casConfig.getServerUrlPrefix() + "/v1/tickets";

            String serviceTicketUrl = getTGT(casServerTicketUrl, userName, password, service);

            if (StringUtils.isEmpty(serviceTicketUrl)) {
                msgResult.setMessage("用户名或密码错误!");
                msgResult.setResult(false);
            } else {
                Map<String, Object> dataMap = new HashMap<>();

                //获取TGT
                String tgt = serviceTicketUrl.substring(casServerTicketUrl.length() + 1);

                session.setAttribute("tgt",tgt);

                // 采用st只能显示单应用登录
//                String st = getST(serviceTicketUrl, service);
                String serviceEnc = "";

                try {
                    serviceEnc = URLEncoder.encode(service, "UTF-8");
                } catch (Exception e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }

                //将成功调用云平台登录接口的用户信息更新
                CompanyUser companyUser = companyUserService.findByUserName(userName);

                if(companyUser!=null) {
                    companyUser.setPassword(des3.encrypt(DES3.DEFAULT_KEY, password));
                    companyUser.setUpdateTime(new Date());
                    companyUser.setUpdateBy(userName);

                    companyUserService.update(companyUser);
                }
                else{
                    companyUser = new CompanyUser();
                    companyUser.setId(UUID.randomUUID().toString());
                    companyUser.setUserName(userName);
                    companyUser.setPassword(des3.encrypt(DES3.DEFAULT_KEY, password));

                    companyUser.setDelFlag(false);
                    companyUser.setCreateTime(new Date());

                    //修复历史数据
                    Company company = companyService.findByCreateBy(userName);

                    if (company!=null){
                        companyUser.setCompanyId(company.getId());
                    }

                    companyUserService.insert(companyUser);
                }

                /**
                 * 注意
                 * 1.cas-server中ticketExpirationPolicies.xml中时间要设长，否则手工测试时，会失效
                 * 2.这里redirect的url，要与service一致
                 * 3.如果是前后端分离的应用，可以要前端先通过ajax请求该方法获取serviceEnc和tgt然后做跳转，就可以在客户端写入tgc
                 */
//                dataMap.put("redirectUrl", service + "?ticket=" + st);
                dataMap.put("redirectUrl",casConfig.getServerLoginUrl() + "?service=" + serviceEnc + "&tgt=" + tgt);
                dataMap.put("ticket", tgt);

                msgResult.setData(dataMap);
                msgResult.setResult(true);
            }
        }
        catch (Exception ex){
            msgResult.setResult(false);
            msgResult.setMessage( ex.getMessage());
        }

        return  msgResult;
    }

    /**
     * cas登录后会带st返回该url，根据st 拦截器会自动到cas服务端注册后再次返回该url，
     * 这时就可以正常访问前端，前端和后端需要处于同一域下，以共享cookie。
     * @return
     */
    @GetMapping(value="/auth/portal")
    @ApiOperation(value="返回前端")
    public String portal(){
        return "redirect:/portal/";
    }

    @GetMapping(value="/auth/userInfo")
    @ResponseBody
    @CrossOrigin
    @ApiOperation(value="获取用户信息")
    public MessageResult<Map> userInfo(HttpServletRequest request, HttpServletResponse response) {
        MessageResult<Map> msgResult = new MessageResult<>();

        try {
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

            Map<String, Object> map = new HashMap<>();

            //{"serialNumber":"000101000178","phone":"13986703087","userName":"2142006","userId":"178"}
            for (String key : principal.getAttributes().keySet()) {
                map.put(key, principal.getAttributes().get(key));
            }

            msgResult.setData(map);
            msgResult.setResult(true);
        }
        catch(Exception ex){
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @GetMapping("/logout")
    @ResponseBody
    @ApiOperation(value="退出")
    public MessageResult<String> logout(HttpSession session,HttpServletRequest request,HttpServletResponse response){
        MessageResult<String> msgResult = new MessageResult<>();

        String tgt = (String)session.getAttribute("tgt");
        String casServerTicketUrl = casConfig.getServerUrlPrefix() + "/v1/tickets";

        try {
//            if(StringUtils.isNotEmpty(tgt)) {
//                HttpClient client = new HttpClient();
//                DeleteMethod del = new DeleteMethod(casServerTicketUrl + "/" + tgt);
//
//                client.executeMethod(del);
//                String html = del.getResponseBodyAsString();
//
//                System.out.println(html);
//            }

            session.removeAttribute(AbstractCasFilter.CONST_CAS_ASSERTION);
            session.invalidate();

            String service = casConfig.getClientHostUrl() + request.getContextPath() + "/portal/";

            msgResult.setData(casConfig.getServerUrlPrefix() + "/logout?service=" + URLEncoder.encode(service,"UTF-8"));
//            Cookie[] cookies = request.getCookies();
//
//            for (Cookie cookie : cookies) {
//                cookie.setMaxAge(0);
//                response.addCookie(cookie);
//            }

            msgResult.setResult(true);
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();

            msgResult.setResult(false);
            msgResult.setMessage(e.getMessage());
        }

        return msgResult;
    }

    @PostMapping(value="/auth/whetherOpenid")
    @ApiOperation(value = "是否绑定了微信")
    @ResponseBody
    public MessageResult<Boolean> whetherOpenid(HttpServletRequest request){
        MessageResult<Boolean> messageResult = new MessageResult<>();

        try {
            Boolean bl = false;
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
            if(StringUtils.isNotEmpty(companyUser.getOpenId())){
                bl = true;
            }

            messageResult.setResult(true);
            messageResult.setData(bl);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping(value="/auth/clearOpenid")
    @ApiOperation(value = "解绑微信")
    @ResponseBody
    public MessageResult<CompanyUser> clearOpenid(HttpServletRequest request){
        MessageResult<CompanyUser> messageResult = new MessageResult<>();

        try {
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
            companyUser.setOpenId("");
            int affectCount = companyUserService.update(companyUser);

            if(affectCount > 0) {
                messageResult.setResult(true);
                messageResult.setMessage("微信解绑成功");
            }else{
                messageResult.setResult(false);
                messageResult.setMessage("微信解绑失败");
            }
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    private String getTGT(String casServerTicketUrl,String username,String password,String service){
        String url = "";

        HttpClient client = new HttpClient();

        PostMethod post = new PostMethod(casServerTicketUrl);
        post.getParams().setParameter(HttpMethodParams.HTTP_CONTENT_CHARSET,"utf-8");

        post.setRequestBody(
                new NameValuePair[] {
                        new NameValuePair("username", username),
                        new NameValuePair("password", password)
//					new NameValuePair("service", service)
                });

        try {
            client.executeMethod(post);
            String html = post.getResponseBodyAsString();

            //System.out.println(html);
            Document doc = Jsoup.parse(html);

            url = doc.select("form").attr("action");

            System.out.println("TGT=" + url);
        }
        catch(Exception e){
            e.printStackTrace();
        }
        finally{
            post.releaseConnection();
        }

        return url;
    }

    public String getST(String url,String service){
        String serviceTicket = "";

        HttpClient client = new HttpClient();

        PostMethod post = new PostMethod(url);

        post.setRequestBody(
                new NameValuePair[] {
                        new NameValuePair("service", service)
                });

        try {
            client.executeMethod(post);
            serviceTicket = post.getResponseBodyAsString();

            System.out.println("ST=" + serviceTicket);
        }
        catch(Exception e){
            e.printStackTrace();
        }
        finally{
            post.releaseConnection();
        }

        return serviceTicket;
    }

    private void ticketValidate(String serverValidate, String serviceTicket, String service) {
        final HttpClient client = new HttpClient();

        GetMethod post = null;

        try {
            String url = serverValidate+"?"+"ticket="+serviceTicket+"&service="+ URLEncoder.encode(service, "UTF-8");

            System.out.println("url=" + url);

            post = new GetMethod(url);
            client.executeMethod(post);

            final String response = post.getResponseBodyAsString();

            System.out.println(response);
        }
        catch (Exception e) {

        } finally {
            //释放资源
            post.releaseConnection();
        }
    }

    public static void main(String[] args){
     //6q3SHr9BhGfFCiXfJ7sUFw==
        DES3 des3 = new DES3();
        try {
            System.out.println(des3.decrypt(DES3.DEFAULT_KEY, "6q3SHr9BhGfFCiXfJ7sUFw=="));
        }
        catch (Exception ex){
            System.out.println(ex.getMessage());
        }
    }
}
